/**
 * 
 */
package org.processmining.models.simpleprecedencediagram;

import java.awt.Dimension;
import java.util.Arrays;
import java.util.Collection;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.AbstractDirectedGraphNode;
import org.processmining.models.graphbased.directed.utils.GraphIterator;
import org.processmining.models.graphbased.directed.utils.GraphIterator.EdgeAcceptor;
import org.processmining.models.graphbased.directed.utils.GraphIterator.NodeAcceptor;
import org.processmining.models.shapes.Rectangle;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 16, 2009
 */
public abstract class SPDNode extends AbstractDirectedGraphNode {
	private final AbstractDirectedGraph<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>> graph;

	public SPDNode(AbstractDirectedGraph<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>> graph, String label) {
		super();

		// to be deleted
		this.graph = graph;

		getAttributeMap().put(AttributeMap.SHAPE, new Rectangle(true));
		getAttributeMap().put(AttributeMap.SQUAREBB, false);
		getAttributeMap().put(AttributeMap.RESIZABLE, true);
		getAttributeMap().put(AttributeMap.LABEL, label);
		getAttributeMap().put(AttributeMap.SHOWLABEL, true);
		getAttributeMap().put(AttributeMap.SIZE, new Dimension(150, 50));
	}

	public AbstractDirectedGraph<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>> getGraph() {
		return graph;
	}

	/**
	 * get Label of the node
	 */
	public String getLabel() {
		return getAttributeMap().get(AttributeMap.LABEL).toString();
	}

	/**
	 * set Label of the node
	 * 
	 * @param newLabel
	 */
	public void setLabel(String newLabel) {
		getAttributeMap().remove(AttributeMap.LABEL);
		getAttributeMap().put(AttributeMap.LABEL, newLabel);
	}

	public Collection<SPDNode> getVisibleSuccessors() {

		final NodeAcceptor<SPDNode> nodeAcceptor = new NodeAcceptor<SPDNode>() {
			public boolean acceptNode(SPDNode node, int depth) {
				return (depth != 0);
			}
		};

		Collection<SPDNode> nodes = GraphIterator.getDepthFirstSuccessors(this, getGraph(),
				new EdgeAcceptor<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>>() {

					public boolean acceptEdge(SPDEdge<? extends SPDNode, ? extends SPDNode> edge, int depth) {
						return !nodeAcceptor.acceptNode(edge.getSource(), depth);
					}
				}, nodeAcceptor);

		return Arrays.asList(nodes.toArray(new SPDNode[0]));
	}

	public Collection<SPDNode> getVisiblePredecessors() {
		final NodeAcceptor<SPDNode> nodeAcceptor = new NodeAcceptor<SPDNode>() {
			public boolean acceptNode(SPDNode node, int depth) {
				return (depth != 0);
			}
		};

		Collection<SPDNode> transitions = GraphIterator.getDepthFirstPredecessors(this, getGraph(),
				new EdgeAcceptor<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>>() {

					public boolean acceptEdge(SPDEdge<? extends SPDNode, ? extends SPDNode> edge, int depth) {
						return !nodeAcceptor.acceptNode(edge.getTarget(), depth);
					}
				}, nodeAcceptor);
		return Arrays.asList(transitions.toArray(new SPDNode[0]));
	}

}
